#include "xl_sprite.h"

// [eXiang]: added this function for single frame sprite
void LoadSprite(u_char *source, XL_SPRITE *sprite)
{

	// load the TIM from memory to the frame buffer
	LoadTim((TIMHEADER *)source);

	// gather all the info about the TIM
	GsGetTimInfo((TIMHEADER *)source, &(sprite->tim));

	// set the RGB values to reasonable values
	sprite->sprite.r = sprite->sprite.g = sprite->sprite.b = 128;

	// set the x(u) and y(v) offsets of the 65x65 sprite within
	// the TIM image
	sprite->sprite.u = sprite->tim.px;
	sprite->sprite.v = sprite->tim.py;

	sprite->sprite.w = sprite->tim.pw;
	sprite->sprite.h = sprite->tim.ph;

	// set the default value used for single frame sprite
	sprite->total_frame = 1;
	sprite->frame_rate = 0;
	sprite->tick = 0;

	// set the clutid and tpage for the sprite using the
	// values returned from GsTimInfo
	sprite->sprite.clutid = sprite->tim.clutid;
	sprite->sprite.tpage = sprite->tim.tpage;

	// other transformation information
	sprite->sprite.scalex = ONE;
	sprite->sprite.scaley = ONE;
	sprite->sprite.rotate = ONE;
}

void LoadAniSprite(u_char *source, XL_SPRITE *sprite, int total_frame, int frame_rate, int sprite_width, int sprite_height)
{

	// load the TIM from memory to the frame buffer
	LoadTim((TIMHEADER *)source);

	// gather all the info about the TIM
	GsGetTimInfo((TIMHEADER *)source, &(sprite->tim));

	// set the RGB values to reasonable values
	sprite->sprite.r = sprite->sprite.g = sprite->sprite.b = 128;

	// set the x(u) and y(v) offsets of the 65x65 sprite within
	// the TIM image
	sprite->sprite.u = sprite->tim.px;
	sprite->sprite.v = sprite->tim.py;

	sprite->sprite.w = sprite_width;
	sprite->sprite.h = sprite_height;

	sprite->total_frame = total_frame;
	sprite->frame_rate = frame_rate;
	sprite->tick = 0;

	// set the clutid and tpage for the sprite using the
	// values returned from GsTimInfo
	sprite->sprite.clutid = sprite->tim.clutid;
	sprite->sprite.tpage = sprite->tim.tpage;

	// other transformation information
	sprite->sprite.scalex = ONE;
	sprite->sprite.scaley = ONE;
	sprite->sprite.rotate = ONE;

}

void PrepareSprite(XL_SPRITE *sprite)
{
	// add the sprite to the graphics queue
	GsSortSprite (&(sprite->sprite));
}

// [eXiang]: direct placing a sprite to screen
void PlaceSprite(XL_SPRITE *sprite, int x, int y)
{
	sprite->sprite.x = x;
	sprite->sprite.y = y;
}

// [eXiang]: move a sprite on screen by pass in addition x and y
void MoveSprite(XL_SPRITE *sprite, int add_x, int add_y)
{
	sprite->sprite.x += add_x;
	sprite->sprite.y += add_y;
}

// [eXiang]: rotate a sprite by pass in angle ratio
void RotateSprite(XL_SPRITE *sprite, float angle_ratio)
{
	if(angle_ratio < 1)
	{
		sprite->sprite.rotate = ONE * angle_ratio;
	}
}

// [eXiang]: scale a sprite by pass in scale ratio of x and y
void ScaleSprite(XL_SPRITE *sprite, float scale_x_ratio, float scale_y_ratio)
{
	sprite->sprite.scalex = ONE * scale_x_ratio;
	sprite->sprite.scaley = ONE * scale_y_ratio;
}

// [eXiang]: alter the sprite RGB value
void ChangeSpriteRGB(XL_SPRITE *sprite, int red, int green, int blue)
{
	if(red != -1)
	{
		sprite->sprite.r = red;
	}

	if(green != -1)
	{
		sprite->sprite.g = green;
	}

	if(blue != -1)
	{
		sprite->sprite.b = blue;
	}
}

// [eXiang]: alter the sprite RGB(brightness) value
void ChangeSpriteBrightness(XL_SPRITE *sprite, int brightness)
{
	sprite->sprite.r = brightness;
	sprite->sprite.g = brightness;
	sprite->sprite.b = brightness;
}

void GetSpriteLoc(XL_SPRITE *sprite, int *x, int *y)
{
	*x = sprite->sprite.x;
	*y = sprite->sprite.y;
}

void RefreshSpriteFrame(XL_SPRITE *sprite)
{
	sprite->tick++;

	if (sprite->tick == sprite->frame_rate)
	{
		sprite->tick = 0;
		SetSpriteFrame(sprite, 1, 0);
	}
}

void SetSpriteFrame(XL_SPRITE *sprite, int frame, int absolute)
{
	int x_frame, y_frame;

	if (absolute == 1)
	{
		sprite->frame = frame;
	}
	else
	{
		sprite->frame = (sprite->frame + frame) % sprite->total_frame;
	}

	// how many sprite in a row
	x_frame = (sprite->tim.pw / sprite->sprite.w);
	// how many column?
	y_frame = sprite->frame / x_frame;

	sprite->sprite.u = ((sprite->frame % x_frame) * sprite->sprite.w) + sprite->tim.px;
	sprite->sprite.v = (y_frame * sprite->sprite.h) + sprite->tim.py;
}
